/* 
 * $Id: ftp_session.h,v 1.7 2000/03/06 07:28:39 shane Exp $
 *
 * Restrictions:
 *  - Only stream MODE is supported.
 *  - Only "ftp" or "anonymous" accepted as a user.
 */

#ifndef FTP_SESSION_H
#define FTP_SESSION_H

#include <netinet/in.h>
#include <limits.h>
#include "watchdog.h"

/* data representation types supported */
#define TYPE_ASCII  0
#define TYPE_IMAGE  1

/* file structure types supported */
#define STRU_FILE   0
#define STRU_RECORD 1

/* data path chosen */
#define DATA_NONE     0
#define DATA_PORT     1
#define DATA_PASSIVE  2

typedef struct {
    int session_active;
    int data_type;
    int file_structure;

    struct sockaddr_in client_addr;
    struct sockaddr_in server_addr;
    telnet_session_t *telnet_session;
    char dir[PATH_MAX+1];

    int data_channel;
    struct sockaddr_in data_port;
    int server_fd;

    watchdog_t *watchdog;
} ftp_session_t;

void ftp_session_init(ftp_session_t *f, 
                      const struct sockaddr_in *client_addr, 
                      const struct sockaddr_in *server_addr, 
                      telnet_session_t *t,
                      const char *dir);
void ftp_session_drop(ftp_session_t *f, const char *reason);
void ftp_session_run(ftp_session_t *f, watchdog_t *w);
void ftp_session_destroy(ftp_session_t *f);

#endif /* FTP_SESSION_H */

