/*
 * $Id: oftpd.c,v 1.8 2000/03/10 03:52:58 shane Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <pwd.h>
#include <syslog.h>
#include "ftp_listener.h"

/* put our executable name here where everybody can see it */
static const char *exe_name = "oftpd";

static void daemonize();

int main(int argc, char *argv[])
{
    struct passwd *user_info;

    ftp_listener_t ftp_listener;

    /* grab our executable name */
    if (argc > 0) {
        exe_name = argv[0];
    }

    /* verify we're running as root */
    if (geteuid() != 0) {
        fprintf(stderr, "%s: program needs root permission to run\n", exe_name);
        exit(1);
    }

    /* check our command-line arguments */
    if (argc != 3) {
        fprintf(stderr, "%s: syntax error\n", exe_name);
	fprintf(stderr, " Syntax: %s user_name root_directory\n", exe_name);
        exit(1);
    }
    user_info = getpwnam(argv[1]);
    if (user_info == NULL) {
        fprintf(stderr, "%s: invalid user name\n", exe_name);
        exit(1);
    }

    /* become a daemon */
    daemonize();

    /* change to root directory */
    if (chroot(argv[2]) != 0) {
        syslog(LOG_ERR, "error with root directory; %s\n", exe_name, strerror(errno));
        exit(1);
    }
    if (chdir("/") != 0) {
        syslog(LOG_ERR, "error changing directory; %s\n", strerror(errno));
	exit(1);
    }

    /* create our main listener */
    if (!ftp_listener_init(&ftp_listener, NULL, 0, 20, 15*60)) {
        syslog(LOG_ERR, "error initializing FTP listener");
        exit(1);
    }

    /* set user to be as inoffensive as possible */
    setgid(user_info->pw_gid);
    setuid(user_info->pw_uid);

    /* start our listener and hang out */
    if (ftp_listener_start(&ftp_listener)) {
        syslog(LOG_NOTICE, "FTP service started and accepting connections");
        ftp_listener_wait(&ftp_listener);
	syslog(LOG_NOTICE, "FTP service ending");
    } else {
        syslog(LOG_ERR, "error starting FTP service");
        exit(1);
    }

    /* done */
    return 0;
}

static void daemonize()
{
    int fork_ret;
    int max_fd;
    int fd;

    max_fd = sysconf(_SC_OPEN_MAX);
    if (max_fd == -1) {
        fprintf(stderr, "%s: error getting maximum open file; %s\n", exe_name, 
	    strerror(errno));
	exit(1);
    }


    fork_ret = fork();
    if (fork_ret == -1) {
        fprintf(stderr, "%s: error forking; %s\n", exe_name, strerror(errno));
	exit(1);
    }
    if (fork_ret != 0) {
        exit(0);
    }
    if (setsid() == -1) {
        fprintf(stderr, "%s: error creating process group; %s\n", exe_name, strerror(errno));
	exit(1);
    }
    fork_ret = fork();
    if (fork_ret == -1) {
        fprintf(stderr, "%s: error forking; %s\n", exe_name, strerror(errno));
	exit(1);
    }
    if (fork_ret != 0) {
        exit(0);
    }
    for (fd=0; fd<max_fd; fd++) {
        close(fd);
    }

    signal(SIGPIPE, SIG_IGN);   /* some Unix systems SIGPIPE on socket activity */
    openlog(NULL, LOG_NDELAY, LOG_DAEMON);
}



