/*
 * $Id: watchdog.c,v 1.3 2000/03/09 13:10:28 shane Exp $
 */

#include <pthread.h>
#include <unistd.h>
#include <assert.h>
#include "watchdog.h"

static int invariant(const watchdog_t *w);
static void *watcher(void *void_w);

void watchdog_init(watchdog_t *w, pthread_t thread, time_t inactivity)
{
    assert(w != NULL);
    assert(inactivity > 0);

    w->watched_thread = thread;
    w->inactivity = inactivity;
    w->alarm = w->inactivity + time(NULL);
    pthread_mutex_init(&w->mutex, NULL);
    w->watching_thread_active = 1;
    pthread_create(&w->watching_thread, NULL, watcher, w);
}

void watchdog_defer(watchdog_t *w)
{
    assert(invariant(w));

    pthread_mutex_lock(&w->mutex);
    w->alarm = w->inactivity + time(NULL);
    pthread_mutex_unlock(&w->mutex);

    assert(invariant(w));
}

void watchdog_destroy(watchdog_t *w)
{
    assert(invariant(w));

    pthread_mutex_lock(&w->mutex);
    if (w->watching_thread_active) {
        pthread_cancel(w->watching_thread);
        w->watching_thread_active = 0;
    }
    pthread_mutex_unlock(&w->mutex);

    assert(invariant(w));
}

static void *watcher(void *void_w)
{
    time_t now;
    watchdog_t *w;

    /* allow us to be cancelled at any time */
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    w = (watchdog_t *)void_w;
    for (;;) {
        sleep(1);
	time(&now);
        pthread_mutex_lock(&w->mutex);
	if (difftime(now, w->alarm) > 0) {
	    pthread_cancel(w->watched_thread);
            w->watching_thread_active = 0;
            pthread_mutex_unlock(&w->mutex);
	    pthread_exit(NULL);
	}
        pthread_mutex_unlock(&w->mutex);
    }
}

static int invariant(const watchdog_t *w)
{
    if (w == NULL) {
        return 0;
    }
    if (w->alarm < 0) {
        return 0;
    }
    return 1;
}
                                       
